Index: nginx/src/http/ngx_http_request.c
===================================================================
--- nginx/src/http/ngx_http_request.c	(revision 4696)
+++ nginx/src/http/ngx_http_request.c	(working copy)
@@ -29,6 +29,8 @@
     ngx_table_elt_t *h, ngx_uint_t offset);
 static ngx_int_t ngx_http_process_cookie(ngx_http_request_t *r,
     ngx_table_elt_t *h, ngx_uint_t offset);
+static ngx_int_t ngx_http_process_x_forwarded_for(ngx_http_request_t *r,
+    ngx_table_elt_t *h, ngx_uint_t offset);
 
 static ngx_int_t ngx_http_process_request_header(ngx_http_request_t *r);
 static void ngx_http_process_request(ngx_http_request_t *r);
@@ -141,7 +143,7 @@
 #if (NGX_HTTP_PROXY || NGX_HTTP_REALIP || NGX_HTTP_GEO)
     { ngx_string("X-Forwarded-For"),
                  offsetof(ngx_http_headers_in_t, x_forwarded_for),
-                 ngx_http_process_header_line },
+                 ngx_http_process_x_forwarded_for },
 #endif
 
 #if (NGX_HTTP_REALIP)
@@ -1542,7 +1544,45 @@
     return NGX_ERROR;
 }
 
+static ngx_int_t
+ngx_http_process_x_forwarded_for(ngx_http_request_t *r,
+    ngx_table_elt_t *h, ngx_uint_t offset)
+{
+    ngx_table_elt_t  **ph;
+    ngx_str_t        value;
+    u_char           *pos;
 
+    ph = (ngx_table_elt_t **) ((char *) &r->headers_in + offset);
+
+    if (*ph == NULL) {
+        *ph = h;
+        value.len = h->value.len;
+        value.data = ngx_pstrdup(r->pool, &(h->value));
+        h->value = value;
+        return NGX_OK;
+    }
+
+    /* Per RFC 2616, concatenate with ", " */
+    value.len = h->value.len + (*ph)->value.len + 2;
+    value.data = ngx_palloc(r->pool, value.len);
+    if(value.data == NULL) {
+        ngx_log_error(NGX_LOG_INFO, r->connection->log, 0,
+                "could not allocate memory while concatenating "
+                "X-Forwarded-For headers.");
+        ngx_http_finalize_request(r, NGX_HTTP_BAD_REQUEST);
+
+        return NGX_ERROR;
+    }
+
+    pos = ngx_cpymem(value.data, (*ph)->value.data, (*ph)->value.len);
+    pos = ngx_cpymem(pos, ", ", 2);
+    ngx_memcpy(pos, h->value.data, h->value.len);
+    ngx_pfree(r->pool, &((*ph)->value));
+    (*ph)->value = value;
+    return NGX_OK;
+}
+
+
 static ngx_int_t
 ngx_http_process_request_header(ngx_http_request_t *r)
 {
